/*
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>

#include "cap32.h"
#include "video.h"
#include "psp_sdl.h"
#include "psp_fmgr.h"
#include "psp_kbd.h"

extern SDL_Surface *back_surface;
extern video_plugin* vid_plugin;

# define PSP_MENU_BORDER_COLOR     psp_sdl_rgb(0x80,0x80,0xF0)
# define PSP_MENU_WARNING_COLOR    psp_sdl_rgb(0xFF,0x00,0x00)
# define PSP_MENU_NOTE_COLOR       psp_sdl_rgb(0xFF,0xFF,0x00)
# define PSP_MENU_BACKGROUND_COLOR psp_sdl_rgb(0x00,0x00,0x00)
# define PSP_MENU_BLACK_COLOR      psp_sdl_rgb(0x00,0x00,0x00)
# define PSP_MENU_AUTHOR_COLOR     psp_sdl_rgb(0x00,0x00,0xFF)

# define PSP_MENU_TEXT_COLOR       psp_sdl_rgb(0x80,0x80,0x80)
# define PSP_MENU_SEL_COLOR        psp_sdl_rgb(0x00,0xff,0xff)

# define PSP_MENU_MIN_TIME         150000

  typedef struct menu_item_t {
    char *title;
  } menu_item_t;

# define MENU_SOUND  0
# define MENU_FPS    1
# define MENU_CLOCK  2

# define MENU_LOAD_SNA   3
# define MENU_LOAD_DSK   4
# define MENU_LOAD_KBD   5

# define MENU_RESET      6
# define MENU_RESET_KBD  7
# define MENU_BACK       8

# define MAX_MENU_ITEM (MENU_BACK + 1)


  static menu_item_t menu_list[] =
  { 
    "Sound enable        :",
    "FPS display         :",
    "Clock frequency     :",

    "Load Snapshot",
    "Load Disk",     
    "Load Keyboard", 

    "Reset CPC",
    "Reset Keyboard",
    "Back to CPC",
  };

  static int cur_menu_id = 0;

  static int cpc_snd_enabled       = 0;
  static int cpc_scr_fps           = 0;
  static int psp_cpu_clock         = 222;

static void 
psp_display_screen_menu(void)
{
  char buffer[32];
  int menu_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  psp_sdl_clear_screen( PSP_MENU_BACKGROUND_COLOR );

  psp_sdl_draw_rectangle(10,10,459,249,PSP_MENU_BORDER_COLOR,0);
  psp_sdl_draw_rectangle(11,11,457,247,PSP_MENU_BORDER_COLOR,0);

  psp_sdl_fill_print( 30, 6, " LTrigger: EXIT ", 
                     PSP_MENU_NOTE_COLOR,  PSP_MENU_BACKGROUND_COLOR);

  psp_sdl_fill_print(30, 254, " []: Cancel  O/X: Valid  SELECT: Back ", 
                     PSP_MENU_BORDER_COLOR, PSP_MENU_BACKGROUND_COLOR);
  psp_sdl_fill_print(370, 254, " By Zx-81 ",
                     PSP_MENU_AUTHOR_COLOR, PSP_MENU_BACKGROUND_COLOR);
  
  x      = 20;
  y      = 25;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    if (cur_menu_id == menu_id) color = PSP_MENU_SEL_COLOR;
    else                        color = PSP_MENU_TEXT_COLOR;
    psp_sdl_fill_print(x, y, menu_list[menu_id].title, color, PSP_MENU_BACKGROUND_COLOR);

    if (menu_id == MENU_SOUND) {
      if (cpc_snd_enabled) strcpy(buffer,"yes");
      else                 strcpy(buffer,"no ");
      psp_sdl_fill_print(210, y, buffer, color, PSP_MENU_BACKGROUND_COLOR);
    } else
    if (menu_id == MENU_FPS) {
      if (cpc_scr_fps) strcpy(buffer,"on ");
      else             strcpy(buffer,"off");
      psp_sdl_fill_print(210, y, buffer, color, PSP_MENU_BACKGROUND_COLOR);
    } else
    if (menu_id == MENU_CLOCK) {
      sprintf(buffer,"%d", psp_cpu_clock);
      psp_sdl_fill_print(210, y, buffer, color, PSP_MENU_BACKGROUND_COLOR);

      y += 2 * y_step;

    }

    y += y_step;
  }
}

static void
psp_main_menu_reset(void)
{
  /* Reset ! */
  psp_display_screen_menu();
  psp_sdl_fill_print(188, 132, "RESET CPC !", 
                     PSP_MENU_WARNING_COLOR, PSP_MENU_BACKGROUND_COLOR);
  psp_sdl_flip();
  emulator_reset(false);
  sleep(2);
}

static void
psp_main_menu_reset_kbd(void)
{
  psp_display_screen_menu();
  psp_sdl_fill_print(170, 132, "RESET KEYBOARD !", 
                     PSP_MENU_WARNING_COLOR, PSP_MENU_BACKGROUND_COLOR);
  psp_sdl_flip();
  psp_kbd_reset_mapping();
  sleep(2);
}

static void
psp_main_menu_clock(void)
{
  if (psp_cpu_clock == 222) {
    psp_cpu_clock = 300;
  } else if (psp_cpu_clock == 300) {
    psp_cpu_clock = 333;
  } else {
    psp_cpu_clock = 222;
  }
}


static void
psp_main_menu_load(int format)
{
  int ret;

  ret = psp_fmgr_menu(format);
  if (ret ==  1) /* load OK */
  {
    psp_display_screen_menu();
    psp_sdl_fill_print(170, 132, "File loaded !", 
                       PSP_MENU_NOTE_COLOR, PSP_MENU_BACKGROUND_COLOR);
    psp_sdl_flip();
    sleep(2);
  }
  else 
  if (ret == -1) /* Load Error */
  {
    psp_display_screen_menu();
    psp_sdl_fill_print(170, 132, "Can't load file !", 
                       PSP_MENU_WARNING_COLOR, PSP_MENU_BACKGROUND_COLOR);
    psp_sdl_flip();
    sleep(2);
  }
}

int 
psp_main_menu(void)
{
  long new_pad;
  long old_pad;
  int  last_time;
  int  end_menu;

  audio_pause();

  psp_kbd_wait_no_button();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

  cpc_snd_enabled       = CPC.snd_enabled;
  cpc_scr_fps           = CPC.scr_fps;
  psp_cpu_clock         = CPC.psp_cpu_clock;

  while (! end_menu)
  {
    psp_display_screen_menu();
    psp_sdl_flip();

    SceCtrlData c;
    sceCtrlReadBufferPositive(&c, 1);

    new_pad = c.Buttons;

    if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_MENU_MIN_TIME)) {
      last_time = c.TimeStamp;
      old_pad = new_pad;

    } else continue;

    if ((c.Buttons & PSP_CTRL_LTRIGGER) == PSP_CTRL_LTRIGGER) {
      /* Exit ! */
      SDL_Quit();
      exit(0);
    } else
    if ((new_pad & PSP_CTRL_CROSS ) || 
        (new_pad & PSP_CTRL_CIRCLE) || 
        (new_pad & PSP_CTRL_RIGHT )) {

      switch (cur_menu_id ) 
      {
        case MENU_SOUND  : cpc_snd_enabled = ! cpc_snd_enabled;
        break;
        case MENU_FPS    : cpc_scr_fps     = ! cpc_scr_fps;
        break;
        case MENU_CLOCK  : psp_main_menu_clock();
        break;

        case MENU_LOAD_SNA : psp_main_menu_load(FMGR_FORMAT_SNA);
                             old_pad = new_pad = 0;
        break;
        case MENU_LOAD_DSK : psp_main_menu_load(FMGR_FORMAT_DSK);
                             old_pad = new_pad = 0;
        break;
        case MENU_LOAD_KBD : psp_main_menu_load(FMGR_FORMAT_KBD);
                             old_pad = new_pad = 0;
        break;
        case MENU_RESET     : psp_main_menu_reset();
        break;
        case MENU_RESET_KBD : psp_main_menu_reset_kbd();
        break;
        case MENU_BACK   : end_menu = 1;
        break;
      }

    } else
    if(new_pad & PSP_CTRL_UP) {
      if (cur_menu_id > 0) cur_menu_id--;
    } else
    if(new_pad & PSP_CTRL_DOWN) {
      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
    } else  
    if(new_pad & PSP_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if(new_pad & PSP_CTRL_SELECT) {
      /* Back to CPC */
      end_menu = 1;
    }
  }
 
  if (end_menu > 0) {
    /* Validate */
    CPC.snd_enabled       = cpc_snd_enabled;
    CPC.scr_fps           = cpc_scr_fps;
    CPC.psp_cpu_clock     = psp_cpu_clock;

    scePowerSetCpuClockFrequency(CPC.psp_cpu_clock, CPC.psp_cpu_clock, CPC.psp_cpu_clock/2);
  }

  psp_kbd_wait_no_button();

  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();
  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();

  audio_resume();

  return 1;
}

