/*
 *  xtiger port on PSP 
 *
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include "global.h"
#include "psp_pg.h"


 extern unsigned char psp_font[];

//system call
extern void sceDisplayWaitVblankStart();
extern void sceDisplaySetMode(long,long,long);
extern void sceDisplaySetFrameBuf(char *topaddr,long linesize,long pixelsize,long);

//constants
#define    PIXELSIZE  1        //in short
#define    LINESIZE  512        //in short
#define    FRAMESIZE  0x44000      //in byte

//480*272 = 60*38
#define CMAX_X 60
#define CMAX_Y 38

//variables
char *pg_vramtop=(char *)0x04000000;
long pg_screenmode;
long pg_showframe;
long pg_drawframe = 0;



void 
pgWaitVn(unsigned long count)
{
  for (; count>0; --count) {
    sceDisplayWaitVblankStart();
  }
}


void pgWaitV()
{
  sceDisplayWaitVblankStart();
}

char *
pgGetVramAddr(unsigned long x,unsigned long y)
{
  return pg_vramtop+(pg_drawframe?FRAMESIZE:0)+x*PIXELSIZE*2+y*LINESIZE*2+0x40000000;
}


void 
pgLoadBackgroundBmp(const char *filename) 
{
  unsigned char *menu_bg;
  unsigned char *vptr;
  unsigned char *menu_bg_buf;
  unsigned short *g_bgBitmap;
   unsigned short x,y,yy,r,g,b,data;

# if 0 
  pspDebugPrintf("pgLoadBackgroundBmp %s\n", filename );
# endif

  int fd;
  fd = sceIoOpen(filename,PSP_O_RDONLY,0777);
  if(fd>=0){
    menu_bg_buf = malloc( (480*272*3+0x36) * sizeof(char) );
    g_bgBitmap = (unsigned short *)malloc(480 * 272 * sizeof(unsigned short) );
    sceIoRead(fd, menu_bg_buf, 480*272*3+0x36);
    menu_bg = menu_bg_buf + 0x36;
    vptr=(unsigned char*)g_bgBitmap;
    for(y=0; y<272; y++){
      for(x=0; x<480; x++){
        yy = 271 - y;
        r = *(menu_bg + (yy*480 + x)*3 + 2);
        g = *(menu_bg + (yy*480 + x)*3 + 1);
        b = *(menu_bg + (yy*480 + x)*3);
        data = (((b & 0xf8) << 7) | ((g & 0xf8) << 2) | (r >> 3));
        *(unsigned short *)vptr=data;
        vptr+=2;
      }
    }
    sceIoClose(fd);
    free(menu_bg_buf);
     pgBitBlt(0,0,480,272,1,g_bgBitmap);
    free(g_bgBitmap);
  }
}

void 
pgInit(char *BackgroundFileName)
{
  sceDisplaySetMode(0,PG_SCREEN_WIDTH,PG_SCREEN_HEIGHT);
  pgScreenFrame(1,1);
  pgLoadBackgroundBmp(BackgroundFileName);
}

void 
pgFillvram(unsigned long color)
{
  unsigned char *vptr0;    //pointer to vram
  unsigned long i;

  vptr0=(unsigned char *)pgGetVramAddr(0,0);
  for (i=0; i<FRAMESIZE/2; i++) {
    *(unsigned short *)vptr0=color;
    vptr0+=PIXELSIZE*2;
  }
}

void 
pgBitBltTransp(unsigned long x,unsigned long y,unsigned long w,unsigned long h,unsigned long mag,const unsigned short *d, unsigned short transparency) {

  unsigned char *vptr0;    //pointer to vram
  unsigned char *vptr;    //pointer to vram
  unsigned long xx,yy,mx,my;
  const unsigned short *dd;
  
  vptr0=(unsigned char *)pgGetVramAddr(x,y);

    for (yy=0; yy<h; yy++) {
      for (my=0; my<mag; my++) {
        vptr=vptr0;
        dd=d;
        for (xx=0; xx<w; xx++) {
    for (mx=0; mx<mag; mx++) {
      if (*dd != transparency) {
        *(unsigned short *)vptr=*dd;
      }
      vptr+=PIXELSIZE*2;
    }
    dd++;
        }
        vptr0+=LINESIZE*2;
      }
      d+=w;
    }

}


void 
pgBitBlt(unsigned long x,unsigned long y,unsigned long w,unsigned long h,unsigned long mag,const unsigned short *d)
{
  unsigned char *vptr0;    //pointer to vram
  unsigned char *vptr;    //pointer to vram
  unsigned long xx,yy,mx,my;
  const unsigned short *dd;
  
  vptr0=(unsigned char *)pgGetVramAddr(x,y);

  for (yy=0; yy<h; yy++) {
    for (my=0; my<mag; my++) {
      vptr=vptr0;
      dd=d;
      for (xx=0; xx<w; xx++) {
        for (mx=0; mx<mag; mx++) {
    *(unsigned short *)vptr=*dd;
    vptr+=PIXELSIZE*2;
        }
        dd++;
      }
      vptr0+=LINESIZE*2;
    }
    d+=w;
  }
}


void pgScreenFrame(long mode,long frame)
{
  pg_screenmode=mode;
  frame=(frame?1:0);
  pg_showframe=frame;
  if (mode==0) {
    //screen off
    pg_drawframe=frame;
    sceDisplaySetFrameBuf(0,0,0,1);
  } else if (mode==1) {
    //show/draw same
    pg_drawframe=frame;
    sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
  } else if (mode==2) {
    //show/draw different
    pg_drawframe=(frame?0:1);
    sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
  }
}


void pgScreenFlip()
{
  pg_showframe=(pg_showframe?0:1);
  pg_drawframe=(pg_drawframe?0:1);
  sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,0);
}


void 
pgScreenFlipV()
{
  pgWaitV();
  pgScreenFlip();
}

void 
pgFlipDrawFrame()
{
  pg_drawframe=(pg_drawframe?0:1);
}

void 
pgFlipShowFrameV()
{
  pgWaitV();
  pg_showframe=(pg_showframe?0:1);
  sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,0);
}

void 
pgDrawBox(int x, int y, int w, int h, int border, int color) 
{
  unsigned short *vram = (unsigned short*)pgGetVramAddr(x * 16, y * 16);
  unsigned short p;
  int xo, yo;
  for (xo = 0; xo <= w; xo++) {
    for (yo = 0; yo <= h; yo++) {
      if (xo == 0 || xo == w || yo == 0 || yo == h) {
        vram[xo + yo * LINESIZE] =  border;
      } else {
        p = vram[xo + yo * LINESIZE];
        vram[xo + yo * LINESIZE] =  color;
      }
    }
  }
}

void 
pgDrawRectangle(int x, int y, int w, int h, int border, int mode) 
{
  unsigned short *vram = (unsigned short*)pgGetVramAddr(x, y);
  int xo, yo;
  if (mode == PG_XOR) {
    for (xo = 0; xo < w; xo++) {
      vram[xo] ^=  border;
      vram[xo + h * LINESIZE] ^=  border;
    }
    for (yo = 0; yo < h; yo++) {
      vram[yo * LINESIZE] ^=  border;
      vram[w + yo * LINESIZE] ^=  border;
    }
  } else {
    for (xo = 0; xo < w; xo++) {
      vram[xo] =  border;
      vram[xo + h * LINESIZE] =  border;
    }
    for (yo = 0; yo < h; yo++) {
      vram[yo * LINESIZE] =  border;
      vram[w + yo * LINESIZE] =  border;
    }
  }
}

void 
pgFillRectangle(int x, int y, int w, int h, int color, int mode)
{
  unsigned short *vram = (unsigned short*)pgGetVramAddr(x, y);
  int xo, yo;
  if (mode == PG_XOR) {
    for (xo = 0; xo <= w; xo++) {
      for (yo = 0; yo <= h; yo++) {
        if ( ((xo == 0) && ((yo == 0) || (yo == h))) ||
             ((xo == w) && ((yo == 0) || (yo == h))) ) {
          /* Skip corner */
        } else {
          vram[xo + yo * LINESIZE] ^=  color;
        }
      }
    }
  } else {
    for (xo = 0; xo <= w; xo++) {
      for (yo = 0; yo <= h; yo++) {
        vram[xo + yo * LINESIZE] =  color;
      }
    }
  }
}

void 
pgPutPixel(int x, int y, int color) 
{
  unsigned short *vram = (unsigned short*)pgGetVramAddr(x, y);
  vram[0] = color;
}


void 
pgPutChar(int x, int y, int color, int bgcolor, char c, int drawfg, int drawbg)
{
  int cx;
  int cy;
  int b;
  int index;
  unsigned short *vram;

  vram  = (unsigned short*)pgGetVramAddr(x, y);
  index = ((unsigned int)c) * 8;

  for (cy=0; cy<8; cy++) {
    b=0x80;
    for (cx=0; cx<8; cx++) {
      if (psp_font[index] & b) {
        if (drawfg) vram[cx + cy * LINESIZE] = color;
      } else {
        if (drawbg) vram[cx + cy * LINESIZE] = bgcolor;
      }
      b = b >> 1;
    }
    index++;
  }
}

void 
pgPrint(int x,int y,const char *str, int color)
{
  int index;
  int x0 = x;

  for (index = 0; str[index] != '\0'; index++) {
    pgPutChar(x, y, color, 0, str[index], 1, 0);
    x += 8;
    if (x >= (PG_SCREEN_WIDTH - 8)) {
      x = x0; y++;
    }
    if (y >= (PG_SCREEN_HEIGHT - 8)) break;
  }
}

void 
pgFillPrint(int x,int y,const char *str, int color, int bgcolor)
{
  int index;
  int x0 = x;

  for (index = 0; str[index] != '\0'; index++) {
    pgPutChar(x, y, color, bgcolor, str[index], 1, 1);
    x += 8;
    if (x >= (PG_SCREEN_WIDTH - 8)) {
      x = x0; y++;
    }
    if (y >= (PG_SCREEN_HEIGHT - 8)) break;
  }
}
